/******************************************************************************
 
	NAME
 
		DiskChange (drive)
 
	SYNOPSIS
 
		DEVICE/A,
 
	LOCATION
 
		:MorphOS/C
 
	FUNCTION
 
		Tells the filesystem that a diskchange occured. Usually the filesystem
	tries to mount the new hd content then.
 
	INPUTS
 
		DRIVE    --  the drive to alter the buffer allocation of
 
	RESULT
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
 
******************************************************************************/

#include <dos/dos.h>

#include <proto/dos.h>
#include <proto/exec.h>

#include "diskchange_version.h"

static const char version[] = VERSTAG;

#define ARG_TEMPLATE "DEVICE/A"

enum
{
	ARG_DEVICE = 0,
	NOOFARGS
};


ULONG DiskChange(void)
{
	struct Library *SysBase=*((struct Library**) 4);
	struct DosLibrary	*DOSBase;
	IPTR   args[NOOFARGS] = { (IPTR)NULL};
	struct RDArgs *rda;
	ULONG  rc = RETURN_FAIL;
	struct MsgPort *FSPort;

	if ((DOSBase=(struct DosLibrary *)OpenLibrary("dos.library", 50UL)))
	{
		rda = ReadArgs(ARG_TEMPLATE, args, NULL);

		if (rda != NULL)
		{
			STRPTR  device = (STRPTR)args[ARG_DEVICE];

			if ((FSPort=DeviceProc(device)))
			{
				if ((DoPkt(FSPort,ACTION_INHIBIT,DOSTRUE,0,0,0,0))!=DOSFALSE)
				{
					DoPkt(FSPort,ACTION_INHIBIT,DOSFALSE,0,0,0,0);
					rc = RETURN_OK;
				}
				else
				{
					PrintFault(IoErr(),"error while inhibiting the device");
				}
			}
			else
			{
				PrintFault(IoErr(),"error while searching the device");
			}

			FreeArgs(rda);
		}
		else
		{
			PrintFault(IoErr(), "DiskChange");
			rc = RETURN_ERROR;
		}

		CloseLibrary((struct Library *)DOSBase);
	}

	return rc;
}
